<?php

/**
 * Plugin wrapper
 **/
class Styles_Plugin {
	
	/**
	 * Plugin Version
	 *
	 * Holds the current plugin version.
	 *
	 * @var string
	 **/
	var $version = '1.0.14';
	
	/**
	 * Plugin DB version
	 * 
	 * Holds the current plugin database version. 
	 * Not the same as the current plugin version.
	 * 
	 * @var string
	 **/
	var $db_version = '1.0';

	/**
	 * @var Styles_CSS
	 */
	var $css;

	/**
	 * @var Styles_Customize
	 */
	var $customize;

	/**
	 * @var Styles_Admin
	 */
	var $admin;

	/**
	 * @var Styles_Child
	 */
	var $child;

	/**
	 * Class added to body and all selectors
	 *
	 * @var string
	 */
	var $body_class = 'styles';

	public function __construct() {

		require_once dirname( __FILE__ ) . '/styles-helpers.php';

		add_action( 'wp_head', array( $this, 'wp_head' ), 1000 );
		add_filter( 'body_class', array( $this, 'body_class' ) );
		
		add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ), 15 );
		add_action( 'admin_menu', array( $this, 'admin_menu' ), 1 );
		add_action( 'customize_register', array( $this, 'customize_register' ), 1 );
		add_action( 'customize_save_after', array( $this, 'customize_save_after' ) );

	}

	/**
	 * Set up detection for child plugins that follow common patterns
	 */
	public function plugins_loaded() {
		if ( ( !is_a( $this->child, 'Styles_Child') && is_user_logged_in() )
			|| apply_filters( 'styles_force_rebuild', false )
		) {

			require_once dirname( __FILE__ ) . '/styles-child.php';
			require_once dirname( __FILE__ ) . '/styles-child-updatable.php';
			require_once dirname( __FILE__ ) . '/styles-child-theme.php';

			$this->child = new Styles_Child( $this );
		}
	}

	/**
	 * Setup WP Admin user interface
	 */
	public function admin_menu() {
		if ( !is_a( $this->admin, 'Styles_Admin') ) {

			require_once dirname( __FILE__ ) . '/styles-admin.php';

			$this->admin = new Styles_Admin( $this );
		}
	}

	/**
	 * Add settings to WP Customize
	 */
	public function customize_register( $wp_customize = null ) {
		if ( !is_a( $this->customize, 'Styles_Customize') ) {

			require_once dirname( __FILE__ ) . '/styles-control.php';
			require_once dirname( __FILE__ ) . '/styles-customize.php';

			$this->customize = new Styles_Customize( $this );
		}
	}

	/**
	 * Rebuild CSS whenever customizer is re-saved
	 */
	function customize_save_after() {
		$this->init_css();
		$this->css->get_css();
	}

	public function init_css() {
		if ( !is_a( $this->css, 'Styles_CSS') ) {

			require_once dirname( __FILE__ ) . '/styles-control.php';
			require_once dirname( __FILE__ ) . '/styles-css.php';

			$this->css = new Styles_CSS( $this );
		}
	}

	public function get_css() {
		global $wp_customize;

		$css = false;

		if ( empty( $wp_customize ) ) {
			$css = get_option( Styles_Helpers::get_option_key( 'css' ) );
		}

		if ( !empty( $wp_customize ) || empty( $css ) || apply_filters( 'styles_force_rebuild', false ) ) {
			// Rebuild
			$this->init_css();
			return $this->css->get_css();
		}else {
			return $css;
		}
	}

	/**
	 * Output <style> tag in head.
	 *
	 * For dynamic CSS, this is much faster than using a <link> tag, which would reload WordPress.
	 * @see http://stylesplugin.com/our-first-review
	 */
	public function wp_head() {
		echo implode( PHP_EOL, array(
			'',
			'<!-- Styles cached and displayed inline for speed. Generated by http://stylesplugin.com -->',
			'<style type="text/css" id="styles-plugin-css">',
			$this->get_css(),
			'</style>',
			'',
		));
	}

	/**
	 * Add Styles body_class to <body> tag
	 */
	public function body_class( $classes ) {
		$classes[] = $this->body_class;
		return $classes;
	}

}